/*
 * ⚠️⚠️⚠️ THIS FILE WAS SCAFFOLDED BY `@grafana/create-plugin`. DO NOT EDIT THIS FILE DIRECTLY. ⚠️⚠️⚠️
 *
 * In order to extend the configuration follow the steps in .config/README.md
 */

module.exports = {
  endOfLine: 'auto',
  printWidth: 120,
  trailingComma: 'es5',
  semi: true,
  jsxSingleQuote: false,
  singleQuote: true,
  useTabs: false,
  tabWidth: 2,
};
